#include "Python.h"
#include "Numeric/arrayobject.h"
#include "math.h"
#include <stdio.h>
#include "util_ext.h"
const double pi = 3.14159265358979;

//Innermost flux function (using w=z+h)
int _flux_function_wellbalanced(double *q_left, 
								double *q_right,
								double normals, 
								double g, 
								double epsilon,
								double *edgeflux, 
								double *max_speed) {		
		int i;
        double z_in, z_out;
		double flux_in[2], flux_out[2];
		double z_star, w_in, h_in, h_in_star, uh_in, soundspeed_in, u_in;
		double w_out, h_out, h_out_star, uh_out, soundspeed_out, u_out;
		double s_max, s_min, denom;
		
		w_in  = q_left[0];
		uh_in = q_left[1];
		uh_in = uh_in*normals;
        z_in  = q_left[2];
		h_in  = q_left[3];
		u_in  = q_left[4];
		u_in = u_in*normals;
	
		w_out  = q_right[0];
		uh_out = q_right[1];
		uh_out = uh_out*normals;
        z_out  = q_right[2];
		h_out  = q_right[3];
		u_out  = q_right[4];
		u_out = u_out*normals;
		
		//printf("------------------ \n");
		//printf("w_in  = %f \n", w_in);
		//printf("uh_in = %f \n", uh_in);
		//printf("z_in  = %f \n", z_in);
		//printf("h_in  = %f \n", h_in);
		//printf("u_in  = %f \n", u_in);
		//printf("------------------ \n");
		//printf("w_out  = %f \n", w_out);
		//printf("uh_out = %f \n", uh_out);
		//printf("z_out  = %f \n", z_out);
		//printf("h_out  = %f \n", h_out);
		//printf("u_out  = %f \n", u_out);	  
		//////if (z_left-z_right != 0.0 ) printf("z_l - z_r = %f \n",z_left-z_right);
		
        
		z_star = max(z_in, z_out);		              											//equation(7)  
				   
		// Compute speeds in x-direction.
		h_in_star = max(0.0, w_in-z_star);														//equation(8)
		
		//if (h_in_star < epsilon) {
		//	u_in = 0.0; h_in_star = 0.0;
		//}
		//else {
		//	u_in = uh_in/h_in_star;
		//}
		//if (h_in_star < epsilon) {
		//h_in_star = 0.0;
		//}
		
		h_out_star = max(0.0, w_out-z_star);													//equation(9)

		//if (h_out_star < epsilon) {
		//	u_out = 0.0; h_out_star = 0.0; 
		//}
		//else {
		//	u_out = uh_out/h_out_star;
		//}
  
		//printf("u_left = %g u_right = %g \n",u_left, u_right);
		soundspeed_in = sqrt(g*h_in_star);
		soundspeed_out = sqrt(g*h_out_star);
		
		s_max = max(u_in+soundspeed_in, u_out+soundspeed_out);
		if (s_max < 0.0) s_max = 0.0;
	
		s_min = min(u_in-soundspeed_in, u_out-soundspeed_out);
		if (s_min > 0.0) s_min = 0.0;
		
		
		// Flux formulas
		flux_in[0] = u_in*h_in_star;
		flux_in[1] = u_in*u_in*h_in_star + 0.5*g*h_in_star*h_in_star;

		flux_out[0] = u_out*h_out_star;
		flux_out[1] = u_out*u_out*h_out_star + 0.5*g*h_out_star*h_out_star;

		// Flux computation
		denom = s_max-s_min;
		
		//printf("Edge Mass Flux = %f \n", edgeflux[0]);
		//printf("Edge Momentum Flux = %f \n", edgeflux[1]);
		
	    if (denom < epsilon && z_out > z_in) {
			for (i=0; i<2; i++) edgeflux[i] = 0.0;
			// Maximal wavespeed		
			edgeflux[1] = edgeflux[1] + (0.5*g*h_in*h_in - 0.5*g*h_in_star*h_in_star );
			edgeflux[1] *= normals;
			*max_speed = 0.0;
			//printf("Part 1 \n");
		} else if (denom < epsilon) {
			for (i=0; i<2; i++) edgeflux[i] = 0.0;
			// Maximal wavespeed
			*max_speed = 0.0;		
			//printf("Part 2 \n");
		//} else if (w_out < z_in) {
		//	for (i=0; i<2; i++) edgeflux[i] = 0.0;
		//	*max_speed = 0.0;
		} else {
			edgeflux[0] = s_max*flux_in[0] - s_min*flux_out[0];
			edgeflux[0] += s_max*s_min*(h_out_star-h_in_star);                
			edgeflux[0] /= denom;
			edgeflux[1] = s_max*flux_in[1] - s_min*flux_out[1];
			edgeflux[1] += s_max*s_min*(flux_out[0]-flux_in[0]);               
			edgeflux[1] /= denom;
			edgeflux[1] = edgeflux[1] + (0.5*g*h_in*h_in - 0.5*g*h_in_star*h_in_star);
			edgeflux[1] *= normals;
			// Maximal wavespeed
			*max_speed = max(fabs(s_max), fabs(s_min));
			//printf("Part 3 \n");
		}  
		//printf("Edge Mass Flux = %f \n", edgeflux[0]);
		//printf("Edge Momentum Flux = %f \n", edgeflux[1]);

    return 0;		
	}
		
		
	
	
// Computational function for flux computation
double _compute_fluxes_ext_wellbalanced(double timestep,
		double epsilon,
		double g,
		
		long* neighbours,
		long* neighbour_vertices,
		double* normals,
		double* areas,
		
		double* stage_edge_values,
		double* xmom_edge_values,
		double* bed_edge_values,
		double* height_edge_values,
		double* vel_edge_values,
		
		double* stage_boundary_values,
		double* xmom_boundary_values,
		double* bed_boundary_values,
		double* height_boundary_values,
		double* vel_boundary_values,
		
		double* stage_explicit_update,
		double* xmom_explicit_update,
		double* bed_explicit_update,
		double* height_explicit_update,
		double* vel_explicit_update,
		
		int number_of_elements,
		double* max_speed_array) {
		
		double flux[2], ql[5], qr[5], edgeflux[2];
		double max_speed, normal;
		int k, i, ki, n, m, nm=0;
		
		for (k=0; k<number_of_elements; k++) {
			flux[0] = 0.0;
			flux[1] = 0.0;
			
			//printf("==================================== \n");
			//printf("==================================== \n");
			//printf("cell_number = %i \n",k);
			
			for (i=0; i<2; i++) {
				
				//printf("......................... \n");
				//printf("interface = %i \n",i);
				
				ki = k*2+i;
				
				ql[0] = stage_edge_values[ki];
				ql[1] = xmom_edge_values[ki];
				ql[2] = bed_edge_values[ki];
                ql[3] = height_edge_values[ki];
				ql[4] = vel_edge_values[ki];
                
				n = neighbours[ki];
				//printf("neighbours[ki] = %li \n",neighbours[ki]);
				if (n<0) {
					m = -n-1;
					qr[0] = stage_boundary_values[m];
					qr[1] = xmom_boundary_values[m];
					qr[2] = ql[2];																			
					qr[3] = height_edge_values[m]; 														
					qr[4] = vel_edge_values[m];																
                    
				} else {
					m = neighbour_vertices[ki];
					nm = n*2+m;
					qr[0] = stage_edge_values[nm];
					qr[1] = xmom_edge_values[nm];
					qr[2] = bed_edge_values[nm];
					qr[3] = height_edge_values[nm];															
					qr[4] = vel_edge_values[nm];															
				}
				
				//Added condition!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
				//if (qr[0] < ql[2]) {
				//ql[4] = 0.0;
				//vel_edge_values[ki] = 0.0;
				//}				
				//if (qr[2] > ql[2]) {
				//ql[1] = 0.0;
				//ql[4] = 0.0;
				//}
				
				normal = normals[ki];
				_flux_function_wellbalanced(ql, qr, normal, g, epsilon, edgeflux, &max_speed);
				flux[0] -= edgeflux[0];
				flux[1] -= edgeflux[1];
				
				// Update timestep based on edge i and possibly neighbour n
				if (max_speed > epsilon) {
	                // Original CFL calculation
					timestep = min(timestep, 0.5*areas[k]/max_speed); //Here, CFL=1.0 is assumed.
					if (n>=0) {
						timestep = min(timestep, 0.5*areas[n]/max_speed); //Here, CFL=1.0 is assumed. 
					}
				}
            } // End edge i (and neighbour n)
			flux[0] /= areas[k];
			stage_explicit_update[k] = flux[0];
			flux[1] /= areas[k];
			xmom_explicit_update[k] = flux[1];
			
			//printf("......................... \n");
			//printf("areas = %f \n", areas[k]);
			//printf("Mass Flux in this cell = %f \n",flux[0]);
			//printf("Momentum Flux in this cell = %f \n",flux[1]);
			//printf("stage_explicit_update in this cell = %f \n",stage_explicit_update[k]);
			//printf("xmom_explicit_update in this cell = %f \n",xmom_explicit_update[k]);
			
			
			//Keep track of maximal speeds
			max_speed_array[k]=max_speed;
		}
		
		return timestep;	
	}
	
	
	



//=========================================================================
// Python Glue
//=========================================================================
PyObject *compute_fluxes_ext_wellbalanced(PyObject *self, PyObject *args) {
  
    PyArrayObject 
	*neighbours, 
	*neighbour_vertices,
	*normals, 
	*areas,
	
	*stage_edge_values,
	*xmom_edge_values,
	*bed_edge_values,
	*height_edge_values,
	*vel_edge_values,
	
	*stage_boundary_values,
	*xmom_boundary_values,
	*bed_boundary_values,
	*height_boundary_values,
	*vel_boundary_values,
	
	*stage_explicit_update,
	*xmom_explicit_update,
	*bed_explicit_update,
	*height_explicit_update,
	*vel_explicit_update,
	
	*max_speed_array;
    
  double timestep, epsilon, g;
  int number_of_elements;
    
  // Convert Python arguments to C
  if (!PyArg_ParseTuple(args, "dddOOOOOOOOOOOOOOOOOOOiO",
			&timestep,
			&epsilon,
			&g,
			
			&neighbours,
			&neighbour_vertices,
			&normals,
			&areas,
			
			&stage_edge_values,
			&xmom_edge_values,
			&bed_edge_values,
			&height_edge_values,
			&vel_edge_values,
			
			&stage_boundary_values,
			&xmom_boundary_values,
			&bed_boundary_values,
			&height_boundary_values,
			&vel_boundary_values,
			
			&stage_explicit_update,
			&xmom_explicit_update,
			&bed_explicit_update,
			&height_explicit_update,
			&vel_explicit_update,
			
			&number_of_elements,
			&max_speed_array)) {
    PyErr_SetString(PyExc_RuntimeError, "comp_flux_ext_wellbalanced.c: compute_fluxes_ext_wellbalanced could not parse input");
    return NULL;
  }

 
  // Call underlying flux computation routine and update 
  // the explicit update arrays 
  timestep = _compute_fluxes_ext_wellbalanced(timestep,
				 epsilon,
				 g,
				 
				 (long*) neighbours -> data,
				 (long*) neighbour_vertices -> data,
				 (double*) normals -> data,
				 (double*) areas -> data,
				 
				 (double*) stage_edge_values -> data,
				 (double*) xmom_edge_values -> data,
				 (double*) bed_edge_values -> data,
				 (double*) height_edge_values -> data,
				 (double*) vel_edge_values -> data,
				 
				 (double*) stage_boundary_values -> data,
				 (double*) xmom_boundary_values -> data,
				 (double*) bed_boundary_values -> data,
				 (double*) height_boundary_values -> data,
				 (double*) vel_boundary_values -> data,
				 
				 (double*) stage_explicit_update -> data,
				 (double*) xmom_explicit_update -> data,
				 (double*) bed_explicit_update -> data,
				 (double*) height_explicit_update -> data,
				 (double*) vel_explicit_update -> data,
				 
				 number_of_elements,
				 (double*) max_speed_array -> data);

  // Return updated flux timestep
  return Py_BuildValue("d", timestep);
}



//-------------------------------
// Method table for python module
//-------------------------------

static struct PyMethodDef MethodTable[] = {
  {"compute_fluxes_ext_wellbalanced", compute_fluxes_ext_wellbalanced, METH_VARARGS, "Print out"},
  {NULL, NULL}
};

// Module initialisation
void initcomp_flux_ext_wellbalanced(void){
  Py_InitModule("comp_flux_ext_wellbalanced", MethodTable);
  import_array();
}
